# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module containing the meshing utility examples."""

meshing_utility_examples = {
    "add_labels_on_cell_zones": [
        'meshing_session.meshing_utilities.add_labels_on_cell_zones(cell_zone_name_list=["elbow-fluid"], label_name_list=["elbow-1"])',
        'meshing_session.meshing_utilities.add_labels_on_cell_zones(cell_zone_id_list=[87], label_name_list=["87-1"])',
        'meshing_session.meshing_utilities.add_labels_on_cell_zones(cell_zone_name_pattern="*", label_name_list=["cell-1"])',
    ],
    "add_labels_on_edge_zones": [
        'meshing_session.meshing_utilities.add_labels_on_edge_zones(edge_zone_name_list=["symmetry:xyplane:hot-inlet:elbow-fluid:feature.20", "hot-inlet:wall-inlet:elbow-fluid:feature.21"], label_name_list=["20-1", "21-1"])',
        'meshing_session.meshing_utilities.add_labels_on_edge_zones(edge_zone_id_list=[22, 23], label_name_list=["22-1", "23-1"])',
        'meshing_session.meshing_utilities.add_labels_on_edge_zones(edge_zone_name_pattern="cold-inlet*", label_name_list=["26-1"])',
    ],
    "add_labels_on_face_zones": [
        'meshing_session.meshing_utilities.add_labels_on_face_zones(face_zone_name_list=["wall-inlet", "wall-elbow"], label_name_list=["wall-inlet-1", "wall-elbow-1"])',
        'meshing_session.meshing_utilities.add_labels_on_face_zones(face_zone_id_list=[30, 31], label_name_list=["hot-inlet-1", "cold-inlet-1"])',
        'meshing_session.meshing_utilities.add_labels_on_face_zones(face_zone_name_pattern="out*", label_name_list=["outlet-1"])',
    ],
    "clean_face_zone_names": [
        "meshing_session.meshing_utilities.clean_face_zone_names()"
    ],
    "delete_all_sub_domains": [
        "meshing_session.meshing_utilities.delete_all_sub_domains()"
    ],
    "delete_empty_cell_zones": [
        "meshing_session.meshing_utilities.delete_empty_cell_zones(cell_zone_id_list=[87])",
        'meshing_session.meshing_utilities.delete_empty_cell_zones(cell_zone_name_list=["elbow.87"])',
        'meshing_session.meshing_utilities.delete_empty_cell_zones(cell_zone_name_pattern="*")',
    ],
    "delete_empty_edge_zones": [
        "meshing_session.meshing_utilities.delete_empty_edge_zones(edge_zone_id_list=[20, 25, 26])",
        'meshing_session.meshing_utilities.delete_empty_edge_zones("symmetry:xyplane:hot-inlet:elbow-fluid:feature.20", "hot-inlet:wall-inlet:elbow-fluid:feature.21")',
        'meshing_session.meshing_utilities.delete_empty_edge_zones(edge_zone_name_pattern="*")',
    ],
    "delete_empty_face_zones": [
        "meshing_session.meshing_utilities.delete_empty_face_zones(face_zone_id_list=[30, 31, 32])",
        'meshing_session.meshing_utilities.delete_empty_face_zones(face_zone_name_list=["wall-inlet", "wallfluid-new"])',
        'meshing_session.meshing_utilities.delete_empty_face_zones(face_zone_name_pattern="*")',
    ],
    "delete_empty_zones": [
        "meshing_session.meshing_utilities.delete_empty_zones(zone_id_list=[20, 32, 87])",
        'meshing_session.meshing_utilities.delete_empty_zones(zone_name_list=["hotfluid-new", "elbow.87"])',
        'meshing_session.meshing_utilities.delete_empty_zones(zone_name_pattern="*")',
    ],
    "delete_marked_faces_in_zones": [
        "meshing_session.meshing_utilities.delete_marked_faces_in_zones(face_zone_id_list=[30, 31, 32])",
        'meshing_session.meshing_utilities.delete_marked_faces_in_zones(face_zone_name_list=["wall-inlet", "wallfluid-new"])',
        'meshing_session.meshing_utilities.delete_marked_faces_in_zones(face_zone_name_pattern="*")',
    ],
    "merge_cell_zones": [
        "meshing_session.meshing_utilities.merge_cell_zones(cell_zone_id_list=[87])",
        'meshing_session.meshing_utilities.merge_cell_zones(cell_zone_name_list=["elbow-fluid"])',
        'meshing_session.meshing_utilities.merge_cell_zones(cell_zone_name_pattern="*")',
    ],
    "merge_cell_zones_with_same_prefix": [
        'meshing_session.meshing_utilities.merge_cell_zones_with_same_prefix(prefix="elbow")'
    ],
    "merge_cell_zones_with_same_suffix": [
        'meshing_session.meshing_utilities.merge_cell_zones_with_same_suffix(suffix="fluid")'
    ],
    "merge_face_zones": [
        "meshing_session.meshing_utilities.merge_face_zones(face_zone_id_list=[30, 31, 32])",
        'meshing_session.meshing_utilities.merge_face_zones(face_zone_name_pattern="wall*")',
    ],
    "merge_face_zones_of_type": [
        'meshing_session.meshing_utilities.merge_face_zones_of_type(face_zone_type="velocity-inlet", face_zone_name_pattern="*")'
    ],
    "merge_face_zones_with_same_prefix": [
        'meshing_session.meshing_utilities.merge_face_zones_with_same_prefix(prefix="elbow")'
    ],
    "remove_id_suffix_from_face_zones": [
        "meshing_session.meshing_utilities.remove_id_suffix_from_face_zones()"
    ],
    "remove_ids_from_zone_names": [
        "meshing_session.meshing_utilities.remove_ids_from_zone_names(zone_id_list=[30, 31, 32])"
    ],
    "remove_labels_on_cell_zones": [
        'meshing_session.meshing_utilities.remove_labels_on_cell_zones(cell_zone_name_list=["elbow-fluid"], label_name_list=["elbow-1"])',
        'meshing_session.meshing_utilities.remove_labels_on_cell_zones(cell_zone_id_list=[87], label_name_list=["87-1"])',
        'meshing_session.meshing_utilities.remove_labels_on_cell_zones(cell_zone_name_pattern="*", label_name_list=["cell-1"])',
    ],
    "remove_labels_on_edge_zones": [
        'meshing_session.meshing_utilities.remove_labels_on_edge_zones(edge_zone_name_list=["symmetry:xyplane:hot-inlet:elbow-fluid:feature.20"], label_name_list=["20-1"])',
        'meshing_session.meshing_utilities.remove_labels_on_edge_zones(edge_zone_id_list=[22], label_name_list=["22-1"])',
        'meshing_session.meshing_utilities.remove_labels_on_edge_zones(edge_zone_name_pattern="*", label_name_list=["26-1"])',
    ],
    "remove_labels_on_face_zones": [
        'meshing_session.meshing_utilities.remove_labels_on_face_zones(face_zone_name_list=["wall-inlet"], label_name_list=["wall-inlet-1"])',
        'meshing_session.meshing_utilities.remove_labels_on_face_zones(face_zone_id_list=[30], label_name_list=["hot-inlet-1"])',
        'meshing_session.meshing_utilities.remove_labels_on_face_zones(face_zone_name_pattern="*", label_name_list=["wall-elbow-1"])',
    ],
    "rename_edge_zone": [
        'meshing_session.meshing_utilities.rename_edge_zone(zone_id=20, new_name="symmetry:xyplane:hot-inlet:elbow-fluid:feature.20-new")'
    ],
    "rename_face_zone": [
        'meshing_session.meshing_utilities.rename_face_zone(zone_name="symmetry:xyplane:hot-inlet:elbow-fluid:feature.20-new", new_name="symmetry:xyplane:hot-inlet:elbow-fluid:feature.20")',
        'meshing_session.meshing_utilities.rename_face_zone(zone_id=32, new_name="outlet-32")',
        'meshing_session.meshing_utilities.rename_face_zone(zone_name="outlet-32", new_name="outlet")',
    ],
    "rename_face_zone_label": [
        'meshing_session.meshing_utilities.rename_face_zone_label(object_name="elbow-fluid-1", old_label_name="outlet", new_label_name="outlet-new")'
    ],
    "rename_object": [
        'meshing_session.meshing_utilities.rename_object(old_object_name="elbow-fluid", new_object_name="elbow-fluid-1")'
    ],
    "renumber_zone_ids": [
        "meshing_session.meshing_utilities.renumber_zone_ids(zone_id_list=[30, 31, 32], start_number=1)"
    ],
    "replace_cell_zone_suffix": [
        'meshing_session.meshing_utilities.replace_cell_zone_suffix(cell_zone_id_list=[87], old_suffix="fluid", new_suffix="fluid-new", merge=True)',
        'meshing_session.meshing_utilities.replace_cell_zone_suffix(cell_zone_name_list=["elbow-fluid-new"], old_suffix="fluid", new_suffix="fluid-new", merge=True)',
    ],
    "replace_edge_zone_suffix": [
        'meshing_session.meshing_utilities.replace_edge_zone_suffix(edge_zone_id_list=[20], old_suffix="fluid", new_suffix="fluid-new", merge=True)',
        'meshing_session.meshing_utilities.replace_edge_zone_suffix(edge_zone_name_list=["hot-inlet:wall-inlet:elbow-fluid:feature.21"], old_suffix="fluid", new_suffix="fluid-new", merge=True)',
    ],
    "replace_face_zone_suffix": [
        'meshing_session.meshing_utilities.replace_face_zone_suffix(face_zone_id_list=[30, 31, 32], separator="-suffix-", replace_with="-with-", merge=False)',
        'meshing_session.meshing_utilities.replace_face_zone_suffix(face_zone_name_list=["cold-inlet", "hot-inlet"], separator="-suffix-", replace_with="-with-", merge=False)',
    ],
    "replace_label_suffix": [
        'meshing_session.meshing_utilities.replace_label_suffix(object_name_list=["elbow-fluid-1"], separator="-", new_suffix="fluid-new")'
    ],
    "replace_object_suffix": [
        'meshing_session.meshing_utilities.replace_object_suffix(object_name_list=["elbow-fluid"], separator="-", new_suffix="fluid-new")'
    ],
    "set_number_of_parallel_compute_threads": [
        "meshing_session.meshing_utilities.set_number_of_parallel_compute_threads(nthreads=2)"
    ],
    "set_object_cell_zone_type": [
        'meshing_session.meshing_utilities.set_object_cell_zone_type(object_name="elbow-fluid", cell_zone_type="mixed")'
    ],
    "set_quality_measure": [
        'meshing_session.meshing_utilities.set_quality_measure(measure="Aspect Ratio")'
    ],
    "_cell_zones_labels_fdl": [
        "meshing_session.meshing_utilities._cell_zones_labels_fdl()"
    ],
    "_cell_zones_str_fdl": ["meshing_session.meshing_utilities._cell_zones_str_fdl()"],
    "_edge_zones_labels_fdl": [
        "meshing_session.meshing_utilities._edge_zones_labels_fdl()"
    ],
    "_edge_zones_str_fdl": ["meshing_session.meshing_utilities._edge_zones_str_fdl()"],
    "_face_zones_labels_fdl": [
        "meshing_session.meshing_utilities._face_zones_labels_fdl()"
    ],
    "_face_zones_str_fdl": ["meshing_session.meshing_utilities._face_zones_str_fdl()"],
    "_node_zones_labels_fdl": [
        "meshing_session.meshing_utilities._node_zones_labels_fdl()"
    ],
    "_node_zones_str_fdl": ["meshing_session.meshing_utilities._node_zones_str_fdl()"],
    "_object_names_str_fdl": [
        "meshing_session.meshing_utilities._object_names_str_fdl()"
    ],
    "_prism_cell_zones_labels_fdl": [
        "meshing_session.meshing_utilities._prism_cell_zones_labels_fdl()"
    ],
    "_prism_cell_zones_str_fdl": [
        "meshing_session.meshing_utilities._prism_cell_zones_str_fdl()"
    ],
    "_regions_str_fdl": ["meshing_session.meshing_utilities._regions_str_fdl()"],
    "_zone_types_fdl": ["meshing_session.meshing_utilities._zone_types_fdl()"],
    "boundary_zone_exists": [
        "meshing_session.meshing_utilities.boundary_zone_exists(zone_id=31)",
        'meshing_session.meshing_utilities.boundary_zone_exists(zone_name="wall-inlet")',
    ],
    "cell_zone_exists": [
        "meshing_session.meshing_utilities.cell_zone_exists(zone_id=87)",
        'meshing_session.meshing_utilities.cell_zone_exists(zone_name="elbow.87")',
    ],
    "convert_zone_ids_to_name_strings": [
        "meshing_session.meshing_utilities.convert_zone_ids_to_name_strings(zone_id_list=[32, 31])"
    ],
    "convert_zone_name_strings_to_ids": [
        'meshing_session.meshing_utilities.convert_zone_name_strings_to_ids(zone_name_list=["outlet", "cold-inlet"])'
    ],
    "copy_face_zone_labels": [
        "meshing_session.meshing_utilities.copy_face_zone_labels(from_face_zone_id=33, to_face_zone_id=34)"
    ],
    "count_marked_faces": [
        'meshing_session.meshing_utilities.count_marked_faces(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"])',
        'meshing_session.meshing_utilities.count_marked_faces(face_zone_name_pattern="*")',
    ],
    "create_boi_and_size_functions_from_refinement_regions": [
        'meshing_session.meshing_utilities.create_boi_and_size_functions_from_refinement_regions(region_type="hexcore", boi_prefix_string="wall", create_size_function=True)'
    ],
    "dump_face_zone_orientation_in_region": [
        'meshing_session.meshing_utilities.dump_face_zone_orientation_in_region(file_name="facezonetest.txt")'
    ],
    "fill_holes_in_face_zone_list": [
        "meshing_session.meshing_utilities.fill_holes_in_face_zone_list(face_zone_id_list=[30, 31, 32], max_hole_edges=2)",
        'meshing_session.meshing_utilities.fill_holes_in_face_zone_list(face_zone_name_list=["wall-inlet", "wallfluid-new"], max_hole_edges=2)',
        'meshing_session.meshing_utilities.fill_holes_in_face_zone_list(face_zone_name_pattern="wall*", max_hole_edges=2)',
    ],
    "get_adjacent_cell_zones_for_given_face_zones": [
        "meshing_session.meshing_utilities.get_adjacent_cell_zones_for_given_face_zones(face_zone_id_list=[29, 30, 31, 32, 33])",
        'meshing_session.meshing_utilities.get_adjacent_cell_zones_for_given_face_zones(face_zone_name_list=["outlet", "inlet", "wall", "internal"])',
        'meshing_session.meshing_utilities.get_adjacent_cell_zones_for_given_face_zones(face_zone_name_pattern="*")',
    ],
    "get_adjacent_face_zones_for_given_cell_zones": [
        "meshing_session.meshing_utilities.get_adjacent_face_zones_for_given_cell_zones(cell_zone_id_list=[87])",
        'meshing_session.meshing_utilities.get_adjacent_face_zones_for_given_cell_zones(cell_zone_name_list=["elbow-fluid"])',
        'meshing_session.meshing_utilities.get_adjacent_face_zones_for_given_cell_zones(cell_zone_name_pattern="*")',
    ],
    "get_adjacent_interior_and_boundary_face_zones_for_given_cell_zones": [
        "meshing_session.meshing_utilities.get_adjacent_interior_and_boundary_face_zones_for_given_cell_zones(cell_zone_id_list=[87])",
        'meshing_session.meshing_utilities.get_adjacent_interior_and_boundary_face_zones_for_given_cell_zones(cell_zone_name_list=["elbow-fluid"])',
        'meshing_session.meshing_utilities.get_adjacent_interior_and_boundary_face_zones_for_given_cell_zones(cell_zone_name_pattern="*")',
    ],
    "get_adjacent_zones_by_edge_connectivity": [
        "meshing_session.meshing_utilities.get_adjacent_zones_by_edge_connectivity(zone_id_list=[29, 30, 31, 32, 33])",
        'meshing_session.meshing_utilities.get_adjacent_zones_by_edge_connectivity(zone_name_list=["outlet", "inlet", "wall", "internal"])',
        'meshing_session.meshing_utilities.get_adjacent_zones_by_edge_connectivity(zone_name_pattern="*")',
    ],
    "get_adjacent_zones_by_node_connectivity": [
        "meshing_session.meshing_utilities.get_adjacent_zones_by_node_connectivity(zone_id_list=[29, 30, 31, 32, 33])",
        'meshing_session.meshing_utilities.get_adjacent_zones_by_node_connectivity(zone_name_list=["outlet", "inlet", "wall", "internal"])',
        'meshing_session.meshing_utilities.get_adjacent_zones_by_node_connectivity(zone_name_pattern="*")',
    ],
    "get_all_objects": ["meshing_session.meshing_utilities.get_all_objects()"],
    "get_average_bounding_box_center": [
        "meshing_session.meshing_utilities.get_average_bounding_box_center(face_zone_id_list=[30, 31, 32])"
    ],
    "get_baffles_for_face_zones": [
        "meshing_session.meshing_utilities.get_baffles_for_face_zones(face_zone_id_list=[29, 30, 31, 32, 33])"
    ],
    "get_bounding_box_of_zone_list": [
        "meshing_session.meshing_utilities.get_bounding_box_of_zone_list(zone_id_list=[26])"
    ],
    "get_cell_mesh_distribution": [
        'meshing_session.meshing_utilities.get_cell_mesh_distribution(cell_zone_id_list=[87], measure="Orthogonal Quality", partitions=2, range=[0.9, 1])',
        'meshing_session.meshing_utilities.get_cell_mesh_distribution(cell_zone_name_list=["elbow-fluid"], measure="Orthogonal Quality", partitions=2, range=[0.9, 1])',
        'meshing_session.meshing_utilities.get_cell_mesh_distribution(cell_zone_name_pattern="*", measure="Orthogonal Quality", partitions=2, range=[0.9, 1])',
    ],
    "get_cell_quality_limits": [
        'meshing_session.meshing_utilities.get_cell_quality_limits(cell_zone_id_list=[87], measure="Orthogonal Quality")',
        'meshing_session.meshing_utilities.get_cell_quality_limits(cell_zone_name_list=["elbow-fluid"], measure="Orthogonal Quality")',
        'meshing_session.meshing_utilities.get_cell_quality_limits(cell_zone_name_pattern="*", measure="Orthogonal Quality")',
    ],
    "get_cell_zone_count": [
        "meshing_session.meshing_utilities.get_cell_zone_count(cell_zone_id_list=[87])",
        'meshing_session.meshing_utilities.get_cell_zone_count(cell_zone_name_list=["elbow-fluid"])',
        'meshing_session.meshing_utilities.get_cell_zone_count(cell_zone_name_pattern="*")',
    ],
    "get_cell_zone_id_list_with_labels": [
        'meshing_session.meshing_utilities.get_cell_zone_id_list_with_labels(cell_zone_id_list=[87], label_name_list=["elbow-1"])',
        'meshing_session.meshing_utilities.get_cell_zone_id_list_with_labels(cell_zone_name_list=["elbow-fluid"], label_name_list=["elbow-1"])',
        'meshing_session.meshing_utilities.get_cell_zone_id_list_with_labels(cell_zone_name_pattern="*", label_name_list=["elbow-1"])',
    ],
    "get_cell_zone_shape": [
        "meshing_session.meshing_utilities.get_cell_zone_shape(cell_zone_id=87)"
    ],
    "get_cell_zone_volume": [
        "meshing_session.meshing_utilities.get_cell_zone_volume(cell_zone_id_list=[87])",
        'meshing_session.meshing_utilities.get_cell_zone_volume(cell_zone_name_list=["elbow-fluid"])',
        'meshing_session.meshing_utilities.get_cell_zone_volume(cell_zone_name_pattern="*")',
    ],
    "get_cell_zones": [
        'meshing_session.meshing_utilities.get_cell_zones(filter="*")',
        "meshing_session.meshing_utilities.get_cell_zones(maximum_entity_count=100)",
        "meshing_session.meshing_utilities.get_cell_zones(xyz_coordinates=[-7, -6, 0.4])",
    ],
    "get_edge_size_limits": [
        "meshing_session.meshing_utilities.get_edge_size_limits(face_zone_id_list=[30, 31, 32])",
        'meshing_session.meshing_utilities.get_edge_size_limits(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"])',
        'meshing_session.meshing_utilities.get_edge_size_limits(face_zone_name_pattern="*")',
    ],
    "get_edge_zone_id_list_with_labels": [
        'meshing_session.meshing_utilities.get_edge_zone_id_list_with_labels(edge_zone_id_list=[20, 21], label_name_list=["20-1", "21-1"])',
        'meshing_session.meshing_utilities.get_edge_zone_id_list_with_labels(edge_zone_name_list=["symmetry:xyplane:hot-inlet:elbow-fluid:feature.20", "hot-inlet:wall-inlet:elbow-fluid:feature.21"], label_name_list=["20-1", "21-1"])',
        'meshing_session.meshing_utilities.get_edge_zone_id_list_with_labels(edge_zone_name_pattern="*", label_name_list=["20-1", "21-1"])',
    ],
    "get_edge_zones": [
        'meshing_session.meshing_utilities.get_edge_zones(filter="*")',
        "meshing_session.meshing_utilities.get_edge_zones(maximum_entity_count=20, only_boundary=False)",
    ],
    "get_edge_zones_list": [
        'meshing_session.meshing_utilities.get_edge_zones_list(filter="*")'
    ],
    "get_edge_zones_of_object": [
        'meshing_session.meshing_utilities.get_edge_zones_of_object(objects=["elbow-fluid"])',
        'meshing_session.meshing_utilities.get_edge_zones_of_object(object_name="elbow-fluid")',
    ],
    "get_embedded_baffles": [
        "meshing_session.meshing_utilities.get_embedded_baffles()"
    ],
    "get_face_mesh_distribution": [
        'meshing_session.meshing_utilities.get_face_mesh_distribution(face_zone_id_list=[30, 31, 32], measure="Orthogonal Quality", partitions=2, range=[0.9, 1])',
        'meshing_session.meshing_utilities.get_face_mesh_distribution(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"], measure="Orthogonal Quality", partitions=2, range=[0.9, 1])',
        'meshing_session.meshing_utilities.get_face_mesh_distribution(face_zone_name_pattern="*", measure="Orthogonal Quality", partitions=2, range=[0.9, 1])',
    ],
    "get_face_quality_limits": [
        'meshing_session.meshing_utilities.get_face_quality_limits(face_zone_id_list=[30, 31, 32], measure="Orthogonal Quality")',
        'meshing_session.meshing_utilities.get_face_quality_limits(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"], measure="Orthogonal Quality")',
        'meshing_session.meshing_utilities.get_face_quality_limits(face_zone_name_pattern="*", measure="Orthogonal Quality")',
    ],
    "get_face_zone_area": [
        "meshing_session.meshing_utilities.get_face_zone_area(face_zone_id_list=[30, 31, 32])",
        'meshing_session.meshing_utilities.get_face_zone_area(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"])',
        'meshing_session.meshing_utilities.get_face_zone_area(face_zone_name_pattern="*")',
    ],
    "get_face_zone_count": [
        "meshing_session.meshing_utilities.get_face_zone_count(face_zone_id_list=[30, 31, 32])",
        'meshing_session.meshing_utilities.get_face_zone_count(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"])',
        'meshing_session.meshing_utilities.get_face_zone_count(face_zone_name_pattern="*")',
    ],
    "get_face_zone_id_list_with_labels": [
        'meshing_session.meshing_utilities.get_face_zone_id_list_with_labels(face_zone_id_list=[33, 34], label_name_list=["wall-inlet-1", "wall-elbow-1"])',
        'meshing_session.meshing_utilities.get_face_zone_id_list_with_labels(face_zone_name_list=["wall-inlet", "wall-elbow"], label_name_list=["wall-inlet-1", "wall-elbow-1"])',
        'meshing_session.meshing_utilities.get_face_zone_id_list_with_labels(face_zone_name_pattern="wall*", label_name_list=["wall-inlet-1", "wall-elbow-1"])',
    ],
    "get_face_zone_node_count": [
        "meshing_session.meshing_utilities.get_face_zone_node_count(face_zone_id=32)",
        'meshing_session.meshing_utilities.get_face_zone_node_count(face_zone_name="outlet")',
    ],
    "get_face_zones": [
        'meshing_session.meshing_utilities.get_face_zones(filter="*")',
        'meshing_session.meshing_utilities.get_face_zones(prism_control_name="*")',
        "meshing_session.meshing_utilities.get_face_zones(xyz_coordinates=[1.4, 1.4, 1.4])",
        "meshing_session.meshing_utilities.get_face_zones(maximum_entity_count=20, only_boundary=True)",
    ],
    "get_face_zones_by_zone_area": [
        "meshing_session.meshing_utilities.get_face_zones_by_zone_area(maximum_zone_area=100)",
        "meshing_session.meshing_utilities.get_face_zones_by_zone_area(minimum_zone_area=10)",
    ],
    "get_face_zones_of_object": [
        'meshing_session.meshing_utilities.get_face_zones_of_object(object_name="elbow-fluid", regions=["elbow-fluid"])',
        'meshing_session.meshing_utilities.get_face_zones_of_object(object_name="elbow-fluid", labels=["outlet"])',
        'meshing_session.meshing_utilities.get_face_zones_of_object(object_name="elbow-fluid", region_type="elbow-fluid")',
        'meshing_session.meshing_utilities.get_face_zones_of_object(object_name="elbow-fluid")',
        'meshing_session.meshing_utilities.get_face_zones_of_object(objects=["elbow-fluid"])',
    ],
    "get_face_zones_with_zone_specific_prisms_applied": [
        "meshing_session.meshing_utilities.get_face_zones_with_zone_specific_prisms_applied()"
    ],
    "get_free_faces_count": [
        "meshing_session.meshing_utilities.get_free_faces_count(face_zone_id_list=[30, 31, 32])",
        'meshing_session.meshing_utilities.get_free_faces_count(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"])',
        'meshing_session.meshing_utilities.get_free_faces_count(face_zone_name_pattern="*")',
    ],
    "get_interior_face_zones_for_given_cell_zones": [
        "meshing_session.meshing_utilities.get_interior_face_zones_for_given_cell_zones(cell_zone_id_list=[87])",
        'meshing_session.meshing_utilities.get_interior_face_zones_for_given_cell_zones(cell_zone_name_list=["elbow-fluid"])',
        'meshing_session.meshing_utilities.get_interior_face_zones_for_given_cell_zones(cell_zone_name_pattern="*")',
    ],
    "get_labels": [
        'meshing_session.meshing_utilities.get_labels(object_name="elbow-fluid")',
        'meshing_session.meshing_utilities.get_labels(object_name="elbow-fluid", filter="*")',
        'meshing_session.meshing_utilities.get_labels(object_name="elbow-fluid", label_name_pattern="*")',
    ],
    "get_labels_on_cell_zones": [
        "meshing_session.meshing_utilities.get_labels_on_cell_zones(cell_zone_id_list=[87])",
        'meshing_session.meshing_utilities.get_labels_on_cell_zones(cell_zone_name_list=["elbow-fluid"])',
        'meshing_session.meshing_utilities.get_labels_on_cell_zones(cell_zone_name_pattern="*")',
    ],
    "get_labels_on_edge_zones": [
        "meshing_session.meshing_utilities.get_labels_on_edge_zones(edge_zone_id_list=[22, 23])",
        'meshing_session.meshing_utilities.get_labels_on_edge_zones(edge_zone_name_list=["symmetry:xyplane:hot-inlet:elbow-fluid:feature.20", "hot-inlet:wall-inlet:elbow-fluid:feature.21"])',
        'meshing_session.meshing_utilities.get_labels_on_edge_zones(edge_zone_name_pattern="cold-inlet*")',
    ],
    "get_labels_on_face_zones": [
        "meshing_session.meshing_utilities.get_labels_on_face_zones(face_zone_id_list=[30, 31])",
        'meshing_session.meshing_utilities.get_labels_on_face_zones(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"])',
        'meshing_session.meshing_utilities.get_labels_on_face_zones(face_zone_name_pattern="out*")',
    ],
    "get_labels_on_face_zones_list": [
        "meshing_session.meshing_utilities.get_labels_on_face_zones_list(face_zone_id_list=[30, 31])"
    ],
    "get_maxsize_cell_zone_by_count": [
        "meshing_session.meshing_utilities.get_maxsize_cell_zone_by_count(zone_id_list=[87])",
        'meshing_session.meshing_utilities.get_maxsize_cell_zone_by_count(zone_name_list=["outlet", "inlet", "wall", "internal"])',
        'meshing_session.meshing_utilities.get_maxsize_cell_zone_by_count(zone_name_pattern="*")',
    ],
    "get_maxsize_cell_zone_by_volume": [
        "meshing_session.meshing_utilities.get_maxsize_cell_zone_by_volume(zone_id_list=[87])",
        'meshing_session.meshing_utilities.get_maxsize_cell_zone_by_volume(zone_name_list=["outlet", "inlet", "wall", "internal"])',
        'meshing_session.meshing_utilities.get_maxsize_cell_zone_by_volume(zone_name_pattern="*")',
    ],
    "get_minsize_face_zone_by_area": [
        "meshing_session.meshing_utilities.get_minsize_face_zone_by_area(zone_id_list=[29, 30, 31, 32, 33])",
        'meshing_session.meshing_utilities.get_minsize_face_zone_by_area(zone_name_list=["outlet", "inlet", "wall", "internal"])',
        'meshing_session.meshing_utilities.get_minsize_face_zone_by_area(zone_name_pattern="*")',
    ],
    "get_minsize_face_zone_by_count": [
        "meshing_session.meshing_utilities.get_minsize_face_zone_by_count(zone_id_list=[29, 30, 31, 32, 33])",
        'meshing_session.meshing_utilities.get_minsize_face_zone_by_count(zone_name_list=["outlet", "inlet", "wall", "internal"])',
        'meshing_session.meshing_utilities.get_minsize_face_zone_by_count(zone_name_pattern="*")',
    ],
    "get_multi_faces_count": [
        "meshing_session.meshing_utilities.get_multi_faces_count(face_zone_id_list=[30, 31, 32])",
        'meshing_session.meshing_utilities.get_multi_faces_count(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"])',
        'meshing_session.meshing_utilities.get_multi_faces_count(face_zone_name_pattern="*")',
    ],
    "get_node_zones": ['meshing_session.meshing_utilities.get_node_zones(filter="*")'],
    "get_objects": [
        'meshing_session.meshing_utilities.get_objects(type_name="mesh")',
        'meshing_session.meshing_utilities.get_objects(filter="*")',
    ],
    "get_overlapping_face_zones": [
        'meshing_session.meshing_utilities.get_overlapping_face_zones(face_zone_name_pattern="*", area_tolerance=0.01, distance_tolerance=0.01)'
    ],
    "get_pairs_of_overlapping_face_zones": [
        "meshing_session.meshing_utilities.get_pairs_of_overlapping_face_zones(face_zone_id_list=[29, 30, 31, 32, 33], join_tolerance=0.001, absolute_tolerance=True, join_angle=45)",
        'meshing_session.meshing_utilities.get_pairs_of_overlapping_face_zones(face_zone_name_list=["outlet", "inlet", "wall", "internal"], join_tolerance=0.001, absolute_tolerance=True, join_angle=45)',
        'meshing_session.meshing_utilities.get_pairs_of_overlapping_face_zones(face_zone_name_pattern="*", join_tolerance=0.001, absolute_tolerance=True, join_angle=45)',
    ],
    "get_prism_cell_zones": [
        "meshing_session.meshing_utilities.get_prism_cell_zones(zone_id_list=[87])",
        'meshing_session.meshing_utilities.get_prism_cell_zones(zone_name_list=["outlet", "inlet", "wall", "internal"])',
        'meshing_session.meshing_utilities.get_prism_cell_zones(zone_name_pattern="*")',
    ],
    "get_region_volume": [
        'meshing_session.meshing_utilities.get_region_volume(object_name="elbow-fluid", sorting_order="ascending")',
        'meshing_session.meshing_utilities.get_region_volume(object_name="elbow-fluid", region_name="elbow-fluid")',
    ],
    "get_regions": [
        'meshing_session.meshing_utilities.get_regions(object_name="elbow-fluid", region_name_pattern="*")',
        'meshing_session.meshing_utilities.get_regions(object_name="elbow-fluid", filter="*")',
        'meshing_session.meshing_utilities.get_regions(object_name="elbow-fluid")',
    ],
    "get_regions_of_face_zones": [
        "meshing_session.meshing_utilities.get_regions_of_face_zones(face_zone_id_list=[29, 30, 31, 32, 33])",
        'meshing_session.meshing_utilities.get_regions_of_face_zones(face_zone_name_list=["outlet", "inlet", "wall", "internal"])',
        'meshing_session.meshing_utilities.get_regions_of_face_zones(face_zone_name_pattern="*")',
    ],
    "get_shared_boundary_face_zones_for_given_cell_zones": [
        "meshing_session.meshing_utilities.get_shared_boundary_face_zones_for_given_cell_zones(cell_zone_id_list=[87])",
        'meshing_session.meshing_utilities.get_shared_boundary_face_zones_for_given_cell_zones(cell_zone_name_list=["elbow-fluid"])',
        'meshing_session.meshing_utilities.get_shared_boundary_face_zones_for_given_cell_zones(cell_zone_name_pattern="*")',
    ],
    "get_tet_cell_zones": [
        "meshing_session.meshing_utilities.get_tet_cell_zones(zone_id_list=[29, 30, 31, 32, 33])",
        'meshing_session.meshing_utilities.get_tet_cell_zones(zone_name_list=["outlet", "inlet", "wall", "internal"])',
        'meshing_session.meshing_utilities.get_tet_cell_zones(zone_name_pattern="*")',
    ],
    "get_unreferenced_cell_zones": [
        "meshing_session.meshing_utilities.get_unreferenced_cell_zones()",
        'meshing_session.meshing_utilities.get_unreferenced_cell_zones(filter="*")',
        'meshing_session.meshing_utilities.get_unreferenced_cell_zones(zone_name_pattern="*")',
    ],
    "get_unreferenced_edge_zones": [
        "meshing_session.meshing_utilities.get_unreferenced_edge_zones()",
        'meshing_session.meshing_utilities.get_unreferenced_edge_zones(filter="*")',
        'meshing_session.meshing_utilities.get_unreferenced_edge_zones(zone_name_pattern="*")',
    ],
    "get_unreferenced_face_zones": [
        "meshing_session.meshing_utilities.get_unreferenced_face_zones()",
        'meshing_session.meshing_utilities.get_unreferenced_face_zones(filter="*")',
        'meshing_session.meshing_utilities.get_unreferenced_face_zones(zone_name_pattern="*")',
    ],
    "get_wrapped_face_zones": [
        "meshing_session.meshing_utilities.get_wrapped_face_zones()"
    ],
    "get_zone_type": [
        "meshing_session.meshing_utilities.get_zone_type(zone_id=87)",
        'meshing_session.meshing_utilities.get_zone_type(zone_name="elbow-fluid")',
    ],
    "get_zones": [
        'meshing_session.meshing_utilities.get_zones(type_name="velocity-inlet")',
        'meshing_session.meshing_utilities.get_zones(group_name="inlet")',
    ],
    "get_zones_with_free_faces_for_given_face_zones": [
        "meshing_session.meshing_utilities.get_zones_with_free_faces_for_given_face_zones(face_zone_id_list=[29, 30, 31, 32, 33])",
        'meshing_session.meshing_utilities.get_zones_with_free_faces_for_given_face_zones(face_zone_name_list=["outlet", "inlet", "wall", "internal"])',
        'meshing_session.meshing_utilities.get_zones_with_free_faces_for_given_face_zones(face_zone_id_list=[face_zone_name_pattern="*"])',
    ],
    "get_zones_with_marked_faces_for_given_face_zones": [
        "meshing_session.meshing_utilities.get_zones_with_marked_faces_for_given_face_zones(face_zone_id_list=[29, 30, 31, 32, 33])",
        'meshing_session.meshing_utilities.get_zones_with_marked_faces_for_given_face_zones(face_zone_name_list=["outlet", "inlet", "wall", "internal"])',
        'meshing_session.meshing_utilities.get_zones_with_marked_faces_for_given_face_zones(face_zone_id_list=[face_zone_name_pattern="*"])',
    ],
    "get_zones_with_multi_faces_for_given_face_zones": [
        "meshing_session.meshing_utilities.get_zones_with_multi_faces_for_given_face_zones(face_zone_id_list=[29, 30, 31, 32, 33])",
        'meshing_session.meshing_utilities.get_zones_with_multi_faces_for_given_face_zones(face_zone_name_list=["outlet", "inlet", "wall", "internal"])',
        'meshing_session.meshing_utilities.get_zones_with_multi_faces_for_given_face_zones(face_zone_id_list=[face_zone_name_pattern="*"])',
    ],
    "interior_zone_exists": [
        "meshing_session.meshing_utilities.interior_zone_exists(zone_id=31)",
        'meshing_session.meshing_utilities.interior_zone_exists(zone_name="wall-inlet")',
    ],
    "mark_bad_quality_faces": [
        "meshing_session.meshing_utilities.mark_bad_quality_faces(face_zone_id_list=[30, 31, 32], quality_limit=0.5, number_of_rings=2)",
        'meshing_session.meshing_utilities.mark_bad_quality_faces(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"], quality_limit=0.5, number_of_rings=2)',
        'meshing_session.meshing_utilities.mark_bad_quality_faces(face_zone_name_pattern="*", quality_limit=0.5, number_of_rings=2)',
    ],
    "mark_duplicate_faces": [
        "meshing_session.meshing_utilities.mark_duplicate_faces(face_zone_id_list=[30, 31, 32])",
        'meshing_session.meshing_utilities.mark_duplicate_faces(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"])',
        'meshing_session.meshing_utilities.mark_duplicate_faces(face_zone_name_pattern="*")',
    ],
    "mark_face_strips_by_height_and_quality": [
        'meshing_session.meshing_utilities.mark_face_strips_by_height_and_quality(face_zone_id_list=[30, 31, 32], strip_type=2, strip_height=2, quality_measure="Size Change", quality_limit=0.5, feature_angle=40)',
        'meshing_session.meshing_utilities.mark_face_strips_by_height_and_quality(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"], strip_type=2, strip_height=2, quality_measure="Size Change", quality_limit=0.5, feature_angle=40)',
        'meshing_session.meshing_utilities.mark_face_strips_by_height_and_quality(face_zone_name_pattern="cold*", strip_type=2, strip_height=2, quality_measure="Size Change", quality_limit=0.5, feature_angle=40)',
    ],
    "mark_faces_by_quality": [
        'meshing_session.meshing_utilities.mark_faces_by_quality(face_zone_id_list=[30, 31, 32], quality_measure="Skewness", quality_limit=0.9, append_marking=False)',
        'meshing_session.meshing_utilities.mark_faces_by_quality(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"], quality_measure="Skewness", quality_limit=0.9, append_marking=False)',
        'meshing_session.meshing_utilities.mark_faces_by_quality(face_zone_name_pattern="*", quality_measure="Skewness", quality_limit=0.9, append_marking=False)',
    ],
    "mark_faces_deviating_from_size_field": [
        'meshing_session.meshing_utilities.mark_faces_deviating_from_size_field(face_zone_id_list=[30, 31, 32], min_size_factor=0.5, max_size_factor=1.1, size_factor_type_to_compare="geodesic")',
        'meshing_session.meshing_utilities.mark_faces_deviating_from_size_field(face_zone_name_list=["cold-inlet", "hot-inlet"] min_size_factor=0.5, max_size_factor=1.1, size_factor_type_to_compare="geodesic")',
        'meshing_session.meshing_utilities.mark_faces_deviating_from_size_field(face_zone_name_pattern="*", min_size_factor=0.5, max_size_factor=1.1, size_factor_type_to_compare="geodesic")',
    ],
    "mark_faces_in_self_proximity": [
        "meshing_session.meshing_utilities.mark_faces_in_self_proximity(face_zone_id_list=[30, 31, 32], relative_tolerance=True, tolerance=0.05, proximity_angle=40.5, ignore_orientation=False)",
        'meshing_session.meshing_utilities.mark_faces_in_self_proximity(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"], relative_tolerance=True, tolerance=0.05, proximity_angle=40.5, ignore_orientation=False)',
        'meshing_session.meshing_utilities.mark_faces_in_self_proximity(face_zone_name_pattern="*", relative_tolerance=True, tolerance=0.05, proximity_angle=40.5, ignore_orientation=False)',
    ],
    "mark_faces_using_node_degree": [
        "meshing_session.meshing_utilities.mark_faces_using_node_degree(face_zone_id_list=[30, 31, 32], node_degree_threshold=2)",
        'meshing_session.meshing_utilities.mark_faces_using_node_degree(face_zone_name_list=["cold-inlet", "hot-inlet"], node_degree_threshold=2)',
        'meshing_session.meshing_utilities.mark_faces_using_node_degree(face_zone_name_pattern="*", node_degree_threshold=2)',
    ],
    "mark_free_faces": [
        "meshing_session.meshing_utilities.mark_free_faces(face_zone_id_list=[30, 31, 32])",
        'meshing_session.meshing_utilities.mark_free_faces(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"])',
        'meshing_session.meshing_utilities.mark_free_faces(face_zone_name_pattern="*")',
    ],
    "mark_invalid_normals": [
        "meshing_session.meshing_utilities.mark_invalid_normals(face_zone_id_list=[30, 31, 32])",
        'meshing_session.meshing_utilities.mark_invalid_normals(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"])',
        'meshing_session.meshing_utilities.mark_invalid_normals(face_zone_name_pattern="*")',
    ],
    "mark_island_faces": [
        "meshing_session.meshing_utilities.mark_island_faces(face_zone_id_list=[30, 31, 32], island_face_count=5)",
        'meshing_session.meshing_utilities.mark_island_faces(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"], island_face_count=5)',
        'meshing_session.meshing_utilities.mark_island_faces(face_zone_name_pattern="cold*", island_face_count=5)',
    ],
    "mark_multi_faces": [
        "meshing_session.meshing_utilities.mark_multi_faces(face_zone_id_list=[30, 31, 32], fringe_length=5)",
        'meshing_session.meshing_utilities.mark_multi_faces(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"], fringe_length=5)',
        'meshing_session.meshing_utilities.mark_multi_faces(face_zone_name_pattern="cold*", fringe_length=5)',
    ],
    "mark_point_contacts": [
        "meshing_session.meshing_utilities.mark_point_contacts(face_zone_id_list=[30, 31, 32])",
        'meshing_session.meshing_utilities.mark_point_contacts(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"])',
        'meshing_session.meshing_utilities.mark_point_contacts(face_zone_name_pattern="cold*")',
    ],
    "mark_self_intersecting_faces": [
        "meshing_session.meshing_utilities.mark_self_intersecting_faces(face_zone_id_list=[30, 31, 32], mark_folded=True)",
        'meshing_session.meshing_utilities.mark_self_intersecting_faces(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"], mark_folded=True)',
        'meshing_session.meshing_utilities.mark_self_intersecting_faces(face_zone_name_pattern="cold*", mark_folded=True)',
    ],
    "mark_sliver_faces": [
        "meshing_session.meshing_utilities.mark_sliver_faces(face_zone_id_list=[30, 31, 32], max_height=2, skew_limit=0.2)",
        'meshing_session.meshing_utilities.mark_sliver_faces(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"], max_height=2, skew_limit=0.2)',
        'meshing_session.meshing_utilities.mark_sliver_faces(face_zone_name_pattern="cold*", max_height=2, skew_limit=0.2)',
    ],
    "mark_spikes": [
        "meshing_session.meshing_utilities.mark_spikes(face_zone_id_list=[30, 31, 32], spike_angle=40.5)",
        'meshing_session.meshing_utilities.mark_spikes(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"], spike_angle=40.5)',
        'meshing_session.meshing_utilities.mark_spikes(face_zone_name_pattern="cold*", spike_angle=40.5)',
    ],
    "mark_steps": [
        "meshing_session.meshing_utilities.mark_steps(face_zone_id_list=[30, 31, 32], step_angle=40.5, step_width=3.3)",
        'meshing_session.meshing_utilities.mark_steps(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"], step_angle=40.5, step_width=3.3)',
        'meshing_session.meshing_utilities.mark_steps(face_zone_name_pattern="cold*", step_angle=40.5, step_width=3.3)',
    ],
    "mesh_check": [
        'meshing_session.meshing_utilities.mesh_check(type_name="face-children", edge_zone_id_list=[22, 23], face_zone_id_list=[30, 31, 32], cell_zone_id_list=[87])',
        'meshing_session.meshing_utilities.mesh_check(type_name="nodes-per-cell", edge_zone_name_pattern="cold-inlet*", face_zone_id_list=[30, 31, 32], cell_zone_id_list=[87])',
        'meshing_session.meshing_utilities.mesh_check(type_name="volume-statistics", edge_zone_id_list=[22, 23], face_zone_name_pattern="*", cell_zone_id_list=[87])',
        'meshing_session.meshing_utilities.mesh_check(type_name="nodes-per-cell", edge_zone_name_pattern="cold-inlet*", face_zone_name_pattern="*", cell_zone_id_list=[87])',
        'meshing_session.meshing_utilities.mesh_check(type_name="face-children", edge_zone_id_list=[22, 23], face_zone_id_list=[30, 31, 32], cell_zone_name_pattern="*")',
        'meshing_session.meshing_utilities.mesh_check(type_name="volume-statistics", edge_zone_name_pattern="cold-inlet*", face_zone_name_pattern="*", cell_zone_name_pattern="*")',
    ],
    "mesh_exists": ["meshing_session.meshing_utilities.mesh_exists()"],
    "print_worst_quality_cell": [
        'meshing_session.meshing_utilities.print_worst_quality_cell(cell_zone_id_list=[87], measure="Orthogonal Quality")',
        'meshing_session.meshing_utilities.print_worst_quality_cell(cell_zone_name_list=["elbow-fluid"], measure="Orthogonal Quality")',
        'meshing_session.meshing_utilities.print_worst_quality_cell(cell_zone_name_pattern="*", measure="Orthogonal Quality")',
    ],
    "project_zone_on_plane": [
        "meshing_session.meshing_utilities.project_zone_on_plane(zone_id=87, plane=[[1, 0, 0], [0, 1, 0], [0, 0, 1]])"
    ],
    "refine_marked_faces_in_zones": [
        "meshing_session.meshing_utilities.refine_marked_faces_in_zones(face_zone_id_list=[30, 31, 32])",
        'meshing_session.meshing_utilities.refine_marked_faces_in_zones(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"])',
        'meshing_session.meshing_utilities.refine_marked_faces_in_zones(face_zone_name_pattern="cold*")',
    ],
    "scale_cell_zones_around_pivot": [
        "meshing_session.meshing_utilities.scale_cell_zones_around_pivot(cell_zone_id_list=[87], scale=[1.1, 1.2, 1.3], pivot=[1.1482939720153809, -2.2965879440307617, 0.7345014897547645], use_bbox_center=True)",
        'meshing_session.meshing_utilities.scale_cell_zones_around_pivot(cell_zone_name_list=["elbow-fluid"], scale=[1.1, 1.2, 1.3], pivot=[1.1482939720153809, -2.2965879440307617, 0.7345014897547645], use_bbox_center=True)',
        'meshing_session.meshing_utilities.scale_cell_zones_around_pivot(cell_zone_name_pattern="*", scale=[1.1, 1.2, 1.3], pivot=[1.1482939720153809, -2.2965879440307617, 0.7345014897547645], use_bbox_center=True)',
    ],
    "scale_face_zones_around_pivot": [
        "meshing_session.meshing_utilities.scale_face_zones_around_pivot(face_zone_id_list=[30, 31, 32], scale=[1.1, 1.2, 1.3], pivot=[1.1482939720153809, -2.2965879440307617, 0.7345014897547645], use_bbox_center=True)",
        'meshing_session.meshing_utilities.scale_face_zones_around_pivot(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"], scale=[1.1, 1.2, 1.3], pivot=[1.1482939720153809, -2.2965879440307617, 0.7345014897547645], use_bbox_center=True)',
        'meshing_session.meshing_utilities.scale_face_zones_around_pivot(face_zone_name_pattern="*", scale=[1.1, 1.2, 1.3], pivot=[1.1482939720153809, -2.2965879440307617, 0.7345014897547645], use_bbox_center=True)',
    ],
    "separate_cell_zone_layers_by_face_zone_using_id": [
        "meshing_session.meshing_utilities.separate_cell_zone_layers_by_face_zone_using_id(cell_zone_id=87, face_zone_id_list=[30, 31, 32], nlayers=2)",
        'meshing_session.meshing_utilities.separate_cell_zone_layers_by_face_zone_using_id(cell_zone_id=87, face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"], nlayers=2)',
        'meshing_session.meshing_utilities.separate_cell_zone_layers_by_face_zone_using_id(cell_zone_id=87, face_zone_name_pattern="*", nlayers=2)',
    ],
    "separate_cell_zone_layers_by_face_zone_using_name": [
        'meshing_session.meshing_utilities.separate_cell_zone_layers_by_face_zone_using_name(cell_zone_name="elbow-fluid", face_zone_id_list=[30, 31, 32], nlayers=2)',
        'meshing_session.meshing_utilities.separate_cell_zone_layers_by_face_zone_using_name(cell_zone_name="elbow-fluid", face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"], nlayers=2)',
        'meshing_session.meshing_utilities.separate_cell_zone_layers_by_face_zone_using_name(cell_zone_name="elbow-fluid", face_zone_name_pattern="*", nlayers=2)',
    ],
    "separate_face_zones_by_cell_neighbor": [
        "meshing_session.meshing_utilities.separate_face_zones_by_cell_neighbor(face_zone_id_list=[30, 31, 32])",
        'meshing_session.meshing_utilities.separate_face_zones_by_cell_neighbor(face_zone_name_list=["cold-inlet", "hot-inlet", "outlet"])',
        'meshing_session.meshing_utilities.separate_face_zones_by_cell_neighbor(face_zone_name_pattern="cold*")',
    ],
    "unpreserve_cell_zones": [
        "meshing_session.meshing_utilities.unpreserve_cell_zones(cell_zone_id_list=[87])",
        'meshing_session.meshing_utilities.unpreserve_cell_zones(cell_zone_name_list=["elbow-fluid"])',
        'meshing_session.meshing_utilities.unpreserve_cell_zones(cell_zone_name_pattern="*")',
    ],
}



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""".. _mixing_tank_workflow:

Mixing Tank Workflow
-------------------------------------------
"""
#############################################################################################################
# Overeview
# ===========================================================================================================
#
# The mixing tank workflow demonstrates basic end to end comprehensive solution for simulating
# mixing tank with an single impeller. This workflow also demonstrates the use of the Python API's
# capabilities to setup complex workflows.
#
# Mixing tanks are commonly used in industries such as chemical processing, food and healthcare,
# pharmaceuticals etc. The mixing tank workflow in ANSYS Fluent allows engineers to analyze the
# performance of mixing tanks, optimize the design of impellers, and understand the flow
# patterns within the tank. By simulating the mixing process, engineers can gain insights
# into the efficiency of mixing, blend time, the distribution of shear forces, and the overall
# performance of the mixing system.
#
# This workflow can be extended to design and develop the vertical application for non-experts
# and seemingly expose the required functionality to democraticize the simulation process.
#
#
# The workflow includes:
#
#  * Importing a geometry file
#  * Generating the mesh
#  * Setting up the solver
#  * Defining the materials
#  * Defining the cell zones and wall boundary conditions
#  * Setting up the solution methods and controls
#  * Defining the report definitions
#  * Running the solver
#  * Post-processing the results
#  * Saving the case file
#  * Closing the session
#
# .. image:: ../../_static/mixing_tank_geom_view.png
#    :align: center
#    :scale: 60%
#    :alt: Mixing Tank Model


#############################################################################################################
# Import required libraries/modules
# ===========================================================================================================

import fnmatch
import platform

import ansys.fluent.core as pyfluent
from ansys.fluent.core import examples

#############################################################################################################
# Launch Fluent session with meshing mode and print Fluent version
# ===========================================================================================================
session = pyfluent.launch_fluent(
    mode="meshing",
    cleanup_on_exit=True,
)
print(session.get_fluent_version())

#############################################################################################################
# Meshing Workflow
# ===========================================================================================================
workflow = session.workflow

filenames = {
    "Windows": "mixing_tank_geom.dsco",
    "Other": "StirredTank.scdoc.pmdb",
}

geometry_filename = examples.download_file(
    filenames.get(platform.system(), filenames["Other"]),
    "pyfluent/examples/MixingTank-WorkFlow",
)

workflow.InitializeWorkflow(WorkflowType="Watertight Geometry")
workflow.TaskObject["Import Geometry"].Arguments = dict(FileName=geometry_filename)
workflow.TaskObject["Import Geometry"].Execute()

workflow.TaskObject["Generate the Surface Mesh"].Execute()
workflow.TaskObject["Describe Geometry"].Arguments = dict(
    SetupType="The geometry consists of only fluid regions with no voids",
    wall_to_internal="Yes",
    InvokeShareTopology="Yes",
)
workflow.TaskObject["Describe Geometry"].Execute()
workflow.TaskObject["Apply Share Topology"].Execute()
workflow.TaskObject["Update Boundaries"].Execute()
workflow.TaskObject["Update Regions"].Execute()
workflow.TaskObject["Add Boundary Layers"].InsertCompoundChildTask()
workflow.TaskObject["smooth-transition_1"].Execute()
workflow.TaskObject["Generate the Volume Mesh"].Execute()

# Write a mesh file for reference
session.tui.file.write_mesh("mixing_tank.msh.h5")

#############################################################################################################
# Switch to the Solver Mode
# ===========================================================================================================
solver = session.switch_to_solver()


###############################################################################################################
# Display the mesh in solver mode
# ===========================================================================================================

# Create a middle plane to display the mesh
y_mid_plane = solver.settings.results.surfaces.plane_surface.create(name="mid_plane")
y_mid_plane.method = "zx-plane"
y_mid_plane.y = 0
y_mid_plane.display()


# Access the graphics object
graphics = solver.settings.results.graphics
# Set the hardcopy format for saving the image
graphics.picture.driver_options.hardcopy_format = "png"

# View settings
geom_view = graphics.views.display_states.create("geom_view")
geom_view.front_faces_transparent = "enable"
geom_view.view_name = "top"

# Get the list of all walls
all_walls = solver.settings.setup.boundary_conditions.wall.get_object_names()
filtered_walls_to_display = [wall for wall in all_walls if "mrf" not in wall]

# Create a mesh object and configure its settings
mesh_object = solver.settings.results.graphics.mesh.create(name="mesh-1")
mesh_object.surfaces_list = filtered_walls_to_display
mesh_object.options.edges = False

mesh_object.display_state_name = geom_view.name()
mesh_object.coloring.option = "manual"
mesh_object.display()
graphics.views.auto_scale()


graphics.picture.save_picture(
    file_name="mixing_tank_geom_view.png",
)


mesh_object.surfaces_list = y_mid_plane.name()
mesh_object.options.edges = True
mesh_object.display()
graphics.views.auto_scale()

graphics.picture.save_picture(
    file_name="mixing_tank_mesh.png",
)

geom_view.front_faces_transparent = "disable"

# %%
# .. image:: ../../_static/mixing_tank_mesh.png
#    :align: center
#    :scale: 60%
#    :alt: Mixing Tank Mesh
#############################################################################################################
# Define Constants
# ===========================================================================================================
g = 9.81  # m/s^2
water_density = 1000.0  # kg/m^3
water_viscosity = 0.001  # kg/m-s
agitation_speed = 10  # rad/s

#############################################################################################################
# Solver Setup and Solve Workflow
# ===========================================================================================================
solver.settings.setup.general.operating_conditions.gravity.enable = True
solver.settings.setup.general.operating_conditions.gravity.components = [0.0, 0.0, -g]

#############################################################################################################
# Define Materials
# ===========================================================================================================
solver.settings.setup.materials.database.copy_by_name(type="fluid", name="water-liquid")
fluid_mat = solver.settings.setup.materials.fluid["water-liquid"]
fluid_mat.density.value = water_density
fluid_mat.viscosity.value = water_viscosity

#############################################################################################################
# Define Cell Zones, Wall Boundary Conditions
# ===========================================================================================================

# Get the list of all Fluid Cell Zones,
fluid_cell_zones = solver.settings.setup.cell_zone_conditions.fluid.get_object_names()
fluid_mrf_cell_zones = [zone for zone in fluid_cell_zones if "mrf" in zone]

for cell_zone in fluid_mrf_cell_zones:
    print(cell_zone)
    # Set the material for each cell zone
    fluid_cell_zone = solver.settings.setup.cell_zone_conditions.fluid[cell_zone]
    fluid_cell_zone.general.material = "water-liquid"
    # Set the reference frame for each cell zone
    fluid_cell_zone.reference_frame.reference_frame_axis_origin = [0, 0, 0.4]
    fluid_cell_zone.reference_frame.reference_frame_axis_direction = [0, 0, -1]
    fluid_cell_zone.reference_frame.frame_motion = True
    # Set the rotation speed for each cell zone
    fluid_cell_zone.reference_frame.mrf_omega.value = agitation_speed

stationary_cell_zones = [zone for zone in fluid_cell_zones if "mrf" not in zone]
stationary_cell_zones = solver.settings.setup.cell_zone_conditions.fluid[
    stationary_cell_zones[0]
]
stationary_cell_zones.general.material = "water-liquid"

# Wall boundary conditions
wall_shaft = [wall for wall in all_walls if "shaft" in wall]

for wall in wall_shaft:
    print(wall)
    wall_boundary = solver.settings.setup.boundary_conditions.wall[wall]
    wall_boundary.momentum.wall_motion = "Moving Wall"
    wall_boundary.momentum.relative = False
    wall_boundary.momentum.rotating = True
    wall_boundary.momentum.rotation_axis_direction = [0, 0, -1]
    wall_boundary.momentum.rotation_speed = agitation_speed


liquid_level_bc = solver.settings.setup.boundary_conditions.wall["wall_liquid_level"]
liquid_level_bc.momentum.wall_motion = "Stationary Wall"
liquid_level_bc.momentum.shear_condition = "Specified Shear"

# Change the Zone type for internal walls to interior
solver.setup.boundary_conditions.set_zone_type(
    zone_list=["fluid_mrf_1-fluid_tank"], new_type="interior"
)

#############################################################################################################
# Solution Methods and Controls
# ===========================================================================================================
solver.settings.solution.methods.p_v_coupling.flow_scheme = "SIMPLE"
solver.settings.solution.methods.spatial_discretization.discretization_scheme[
    "pressure"
] = "presto!"
solver.settings.solution.controls.under_relaxation["pressure"] = 0.5
solver.settings.solution.controls.under_relaxation["mom"] = 0.3
solver.settings.solution.controls.under_relaxation["k"] = 0.6
solver.settings.solution.controls.under_relaxation["omega"] = 0.6
solver.settings.solution.controls.under_relaxation["turb-viscosity"] = 0.6

##############################################################################################################
#  Residuals Criteria & Initialization
# ===========================================================================================================

# Residuals criteria
residuals_options = solver.settings.solution.monitor.residual
residuals_options.equations["continuity"].absolute_criteria = 0.0001
residuals_options.equations["continuity"].monitor = True  # Enable continuity residuals
residuals_options.equations["x-velocity"].absolute_criteria = 0.0001
residuals_options.equations["y-velocity"].absolute_criteria = 0.0001
residuals_options.equations["z-velocity"].absolute_criteria = 0.0001
residuals_options.equations["k"].absolute_criteria = 0.0001
residuals_options.equations["omega"].absolute_criteria = 0.0001


# Initialize the solution
initialization = solver.settings.solution.initialization
initialization.reference_frame = "absolute"
initialization.initialization_type = "standard"
initialization.standard_initialize()


#############################################################################################################
# Define Report Definitions
# ===========================================================================================================

# Create a report definition for the volume average velocity magnitude
volume_avg_vmag_report_def = solver.settings.solution.report_definitions.volume.create(
    "volume-avg-vmag"
)
volume_avg_vmag_report_def.report_type.allowed_values()  # output the allowed values for report type
volume_avg_vmag_report_def.report_type = "volume-average"
volume_avg_vmag_report_def.field = "velocity-magnitude"
volume_avg_vmag_report_def.cell_zones = fluid_cell_zones

# Report plot
volume_avg_vmag_report_plot = solver.settings.solution.monitor.report_plots.create(
    "volume-avg-vmag-rplot"
)
volume_avg_vmag_report_plot.report_defs = "volume-avg-vmag"
# Report file
volume_avg_vmag_report_file = solver.settings.solution.monitor.report_files.create(
    "volume-avg-vmag-rfile"
)
volume_avg_vmag_report_file.report_defs = "volume-avg-vmag"


# Create a report definition for the torque on the impeller walls
torque_report_def = solver.settings.solution.report_definitions.moment.create(
    "torque_imp_walls"
)
torque_report_def.report_output_type.allowed_values()  # output the allowed values for report output type
torque_report_def.report_output_type = "Moment"
filtered_walls = fnmatch.filter(all_walls, "wall_impeller*")
torque_report_def.zones = filtered_walls

# Report plot
torque_report_plot = solver.settings.solution.monitor.report_plots.create(
    "torque_imp_walls_rplot"
)
torque_report_plot.report_defs = "torque_imp_walls"
# Report file
torque_report_file = solver.settings.solution.monitor.report_files.create(
    "torque_imp_walls_rfile"
)
torque_report_file.report_defs = "torque_imp_walls"


# Average and maximum dissipation rate report definitions
average_dissipation_rate_report_def = (
    solver.settings.solution.report_definitions.volume.create(
        "average-dissipation-rate"
    )
)
average_dissipation_rate_report_def.report_type = "volume-average"
average_dissipation_rate_report_def.field = "turb-diss-rate"
average_dissipation_rate_report_def.cell_zones = fluid_cell_zones

maximum_dissipation_rate_report_def = (
    solver.settings.solution.report_definitions.volume.create(
        "maximum-dissipation-rate"
    )
)
maximum_dissipation_rate_report_def.report_type = "volume-max"
maximum_dissipation_rate_report_def.field = "turb-diss-rate"
maximum_dissipation_rate_report_def.cell_zones = fluid_cell_zones

# Average and maximum strain rate report definitions
average_strain_rate_report_def = (
    solver.settings.solution.report_definitions.volume.create("average-strain-rate")
)
average_strain_rate_report_def.report_type = "volume-average"
average_strain_rate_report_def.field = "strain-rate-mag"
average_strain_rate_report_def.cell_zones = fluid_cell_zones


maximum_strain_rate_report_def = (
    solver.settings.solution.report_definitions.volume.create("maximum-strain-rate")
)
maximum_strain_rate_report_def.report_type = "volume-max"
maximum_strain_rate_report_def.field = "strain-rate-mag"
maximum_strain_rate_report_def.cell_zones = fluid_cell_zones

#############################################################################################################
# Run Solver
# ===========================================================================================================

# Run the calculation
run_calculation = solver.settings.solution.run_calculation
run_calculation.iter_count = 500

# Write the case file
solver.settings.file.write(file_type="case", file_name="mixing_tank_final.cas.h5")
# Run the calculation
run_calculation.calculate()

#############################################################################################################
# Post-Processing Workflow
# ===========================================================================================================

# Set View
contour_view = graphics.views.display_states.create("contour_view")
contour_view.front_faces_transparent = "disable"
contour_view.view_name = "top"

# Define the contour for velocity magnitude
velocity_contour = solver.settings.results.graphics.contour.create(
    name="velocity-contour"
)
velocity_contour.field = "velocity-magnitude"
velocity_contour.surfaces_list = ["mid_plane"]
velocity_contour.display_state_name = contour_view.name()
velocity_contour.display()

graphics.views.auto_scale()
graphics.picture.save_picture(file_name="mixing_tank_velocity_contour.png")

# %%
# .. image:: ../../_static/mixing_tank_velocity_contour.png
#    :align: center
#    :scale: 60%
#    :alt: Mixing Tank Model Velocity Contour


#############################################################################################################
# Save the case file
# ===========================================================================================================
solver.settings.file.write(file_type="case-data", file_name="mixing_tank_final.cas.h5")

#############################################################################################################
# Close the session
# ===========================================================================================================
session.exit()


# sphinx_gallery_thumbnail_path = '_static/mixing_tank_velocity_contour.png'



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Client-side service allowing access and modification of rpvars.

The primary interaction with Fluent should not be through low-level
variables like rpvars but instead through the high-level object-based
interfaces: solver settings objects and task-based meshing workflow.
"""

from typing import Any, List

import ansys.fluent.core.filereader.lispy as lispy
from ansys.fluent.core.solver.error_message import allowed_name_error_message


class RPVars:
    """Access to rpvars in a specific session."""

    _allowed_values = None

    def __init__(self, eval_fn):
        """Initialize RPVars."""
        self._eval_fn = eval_fn

    def __call__(self, var: str | None = None, val: Any | None = None) -> Any:
        """Set or get a specific rpvar, or get the full rpvar state.

        Parameters
        ----------
        var : str
            Name of the rpvar. Set the specified rpvar if val is provided,
            or else return its value.
        val : Any
            Value of the rpvar. Set the specified var to this value if val
            is specified.

        Returns
        -------
        Any
            A dict containing the full rpvar state if all arguments are
            unspecified, the value of the rpvar if only var is specified,
            or a string with the var name if both arguments are specified.

        Examples
        --------
        >>> import ansys.fluent.core as pyfluent
        >>> solver = pyfluent.launch_fluent(mode="solver")
        >>> iter_count = 100
        >>> solver.rp_vars("number-of-iterations", iter_count)
        'number-of-iterations'
        >>> solver.rp_vars("number-of-iterations")
        100

        >>> # Get dictionary of all available rpvars:

        >>> solver.rp_vars()
        {'sg-swirl?': False, 'rp-seg?': True, 'rf-energy?': False, 'rp-inviscid?': False, ...
        'number-of-iterations': 100, ...}
        """
        return (
            self._set_var(var, val)
            if val is not None
            else (self._get_var(var) if var is not None else self._get_vars())
        )

    def allowed_values(self) -> List[str]:
        """Returns list with the allowed rpvars names.

        Returns
        -------
        List[str]
            List with all allowed rpvars names.
        """
        if not RPVars._allowed_values:
            RPVars._allowed_values = lispy.parse(
                self._eval_fn("(cx-send '(map car rp-variables))")
            )
        return RPVars._allowed_values

    def _get_var(self, var: str):
        if var not in self.allowed_values():
            raise RuntimeError(
                allowed_name_error_message(
                    context="rp-vars",
                    trial_name=var,
                    allowed_values=RPVars._allowed_values,
                )
            )

        cmd = f"(rpgetvar {RPVars._var(var)})"
        return self._execute(cmd)

    def _get_vars(self):
        list_val = self._execute("(cx-send 'rp-variables)")
        return {val[0]: val[1] for val in list_val}

    def _set_var(self, var: str, val):
        prefix = "'" if isinstance(val, (list, tuple)) else ""
        cmd = f"(rpsetvar {RPVars._var(var)} {prefix}{lispy.to_string(val)})"
        return self._execute(cmd)

    def _execute(self, cmd: str):
        scheme_val = self._eval_fn(cmd)
        return lispy.parse(scheme_val)

    @staticmethod
    def _var(var: str):
        if not var.startswith("'"):
            var = "'" + var
        return var



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Provides a module for launching Fluent in pim mode.

Examples
--------

>>> from ansys.fluent.core.launcher.launcher import create_launcher
>>> from ansys.fluent.core.launcher.launch_options import LaunchMode, FluentMode

>>> pim_meshing_launcher = create_launcher(LaunchMode.PIM, mode=FluentMode.MESHING)
>>> pim_meshing_session = pim_meshing_launcher()

>>> pim_solver_launcher = create_launcher(LaunchMode.PIM)
>>> pim_solver_session = pim_solver_launcher()
"""

import inspect
import logging
import os
from typing import Any, Dict

from ansys.fluent.core.fluent_connection import FluentConnection
from ansys.fluent.core.launcher.launch_options import (
    Dimension,
    FluentLinuxGraphicsDriver,
    FluentMode,
    FluentWindowsGraphicsDriver,
    Precision,
    _get_argvals_and_session,
)
from ansys.fluent.core.session_meshing import Meshing
from ansys.fluent.core.session_pure_meshing import PureMeshing
from ansys.fluent.core.session_solver import Solver
from ansys.fluent.core.session_solver_icing import SolverIcing
from ansys.fluent.core.utils.file_transfer_service import PimFileTransferService
from ansys.fluent.core.utils.fluent_version import FluentVersion
import ansys.platform.instancemanagement as pypim

_THIS_DIR = os.path.dirname(__file__)
_OPTIONS_FILE = os.path.join(_THIS_DIR, "fluent_launcher_options.json")
logger = logging.getLogger("pyfluent.launcher")


class PIMLauncher:
    """Instantiates Fluent session in `PIM <https://pypim.docs.pyansys.com/version/stable/>`_ mode."""

    def __init__(
        self,
        mode: FluentMode | str | None = None,
        graphics_driver: (
            FluentWindowsGraphicsDriver | FluentLinuxGraphicsDriver | str | None
        ) = None,
        product_version: FluentVersion | str | float | int | None = None,
        dimension: Dimension | int | None = None,
        precision: Precision | str | None = None,
        processor_count: int | None = None,
        start_timeout: int = 60,
        additional_arguments: str = "",
        cleanup_on_exit: bool = True,
        start_transcript: bool = True,
        gpu: bool | None = None,
        start_watchdog: bool | None = None,
        file_transfer_service: Any | None = None,
    ):
        """
        Launch a Fluent session in `PIM <https://pypim.docs.pyansys.com/version/stable/>`_ mode.

        Parameters
        ----------
        mode : FluentMode
            Specifies the launch mode of Fluent for targeting a specific session type.
        graphics_driver : FluentWindowsGraphicsDriver or FluentLinuxGraphicsDriver
            Specifies the graphics driver for Fluent. Options are from the ``FluentWindowsGraphicsDriver`` enum
            (for Windows) or the ``FluentLinuxGraphicsDriver`` enum (for Linux).
        product_version : FluentVersion or str or float or int, optional
            Indicates the version of Ansys Fluent to launch. For example, to use version 2025 R1, pass
            any of ``FluentVersion.v251``, ``"25.1.0"``, ``"25.1"``, ``25.1``, or ``251``. Defaults to ``None``,
            which uses the newest installed version.
        dimension : Dimension or int, optional
            Specifies the geometric dimensionality of the Fluent simulation. Defaults to ``None``,
            which corresponds to ``Dimension.THREE``. Acceptable values include ``Dimension.TWO``,
            ``Dimension.THREE``, or integers ``2`` and ``3``.
        precision : Precision or str, optional
            Defines the floating point precision. Defaults to ``None``, which corresponds to
            ``Precision.DOUBLE``. Acceptable values include ``Precision.SINGLE``,
            ``Precision.DOUBLE``, or strings ``"single"`` and ``"double"``.
        processor_count : int, optional
            Specifies the number of processors to use. Defaults to ``None``, which uses 1 processor.
            In job scheduler environments, this value limits the total number of allocated cores.
        start_timeout : int, optional
            Maximum allowable time in seconds for connecting to the Fluent server. Defaults to 60 seconds.
        additional_arguments : str, optional
            Additional command-line arguments for Fluent, formatted as they would be on the command line.
        cleanup_on_exit : bool
            Determines whether to shut down the connected Fluent session upon exit or when calling
            the session's `exit()` method. Defaults to True.
        start_transcript : bool
            Indicates whether to start streaming the Fluent transcript in the client. Defaults to True;
            streaming can be controlled via `transcript.start()` and `transcript.stop()` methods on the session object.
        gpu : bool, optional
            If True, starts Fluent with GPU Solver enabled.
        start_watchdog : bool, optional
            If True and `cleanup_on_exit` is True, an independent watchdog process is run to ensure that any local
            GUI-less Fluent sessions started by PyFluent are properly closed when the current Python process ends.
        file_transfer_service : Any, optional
            Service for uploading/downloading files to/from the server.

        Returns
        -------
        Union[Meshing, PureMeshing, Solver, SolverIcing, dict]
            Session object or configuration dictionary if ``dry_run`` is True.

        Raises
        ------
        UnexpectedKeywordArgument
            If an unexpected keyword argument is provided.

        Notes
        -----
        In job scheduler environments (e.g., SLURM, LSF, PBS), resources and compute nodes are allocated,
        and core counts are queried from these environments before being passed to Fluent.
        """

        if additional_arguments:
            logger.warning(
                "'additional_arguments' option for 'launch_fluent()' method is not supported "
                "when starting a remote Fluent PyPIM client."
            )

        if start_watchdog:
            logger.warning(
                "'start_watchdog' argument for 'launch_fluent()' method is not supported "
                "when starting a remote Fluent PyPIM client."
            )
        locals_ = locals().copy()
        argvals = {
            arg: locals_.get(arg)
            for arg in inspect.getargvalues(inspect.currentframe()).args
        }
        self.argvals, self.new_session = _get_argvals_and_session(argvals)
        self.file_transfer_service = file_transfer_service
        if self.argvals["start_timeout"] is None:
            self.argvals["start_timeout"] = 60

    def __call__(self):
        logger.info("Starting Fluent remotely. The startup configuration is ignored.")
        if self.argvals["product_version"]:
            fluent_product_version = str(
                FluentVersion(self.argvals["product_version"]).number
            )
        else:
            fluent_product_version = None

        return launch_remote_fluent(
            session_cls=self.new_session,
            start_transcript=self.argvals["start_transcript"],
            product_version=fluent_product_version,
            cleanup_on_exit=self.argvals["cleanup_on_exit"],
            mode=self.argvals["mode"],
            dimensionality=self.argvals["dimension"],
            launcher_args=self.argvals,
            file_transfer_service=self.file_transfer_service,
        )


def launch_remote_fluent(
    session_cls,
    start_transcript: bool,
    product_version: str | None = None,
    cleanup_on_exit: bool = True,
    mode: FluentMode = FluentMode.SOLVER,
    dimensionality: str | None = None,
    launcher_args: Dict[str, Any] | None = None,
    file_transfer_service: Any | None = None,
) -> Meshing | PureMeshing | Solver | SolverIcing:
    """Launch Fluent remotely using `PyPIM <https://pypim.docs.pyansys.com>`.

    Ensure that you are in an environment where PyPIM is configured.
    Use the :func:`pypim.is_configured <ansys.platform.instancemanagement.is_configured>`
    method to verify that PyPIM is configured.

    Parameters
    ----------
    session_cls: type(Meshing) | type(PureMeshing) | type(Solver) | type(SolverIcing)
        Session type.
    start_transcript: bool
        Whether to start streaming the Fluent transcript in the client.
    product_version : str, optional
        Version of Ansys Fluent to launch. Default is ``None`` for the newest version.
    cleanup_on_exit : bool, optional
        Whether to clean up and exit Fluent when Python exits. Default is ``True``.
    mode : FluentMode, optional
        Launch Fluent in meshing mode. Default is ``FluentMode.SOLVER``.
    dimensionality : str, optional
        Geometric dimensionality of the Fluent simulation. Default is ``None`` (3D).
    file_transfer_service : optional
        Service for uploading/downloading files to/from the server.
    launcher_args : Any
        Launcher arguments.

    Returns
    -------
    Meshing | PureMeshing | Solver | SolverIcing
        Session object.
    """

    pim = pypim.connect()

    instance = create_fluent_instance(
        pim=pim,
        mode=mode,
        dimensionality=dimensionality,
        product_version=product_version,
    )

    instance.wait_for_ready()

    channel = instance.build_grpc_channel()

    fluent_connection = create_fluent_connection(
        channel=channel,
        cleanup_on_exit=cleanup_on_exit,
        instance=instance,
        launcher_args=launcher_args,
    )

    file_transfer_service = get_file_transfer_service(
        file_transfer_service, fluent_connection
    )

    return session_cls(
        fluent_connection=fluent_connection,
        scheme_eval=fluent_connection._connection_interface.scheme_eval,
        file_transfer_service=file_transfer_service,
        start_transcript=start_transcript,
    )


def create_fluent_instance(
    pim, mode: FluentMode, dimensionality: str | None, product_version: str | None
):
    """Create a Fluent instance based on mode and dimensionality."""

    product_name = (
        "fluent-meshing"
        if FluentMode.is_meshing(mode)
        else "fluent-2ddp" if dimensionality == Dimension.TWO else "fluent-3ddp"
    )

    return pim.create_instance(
        product_name=product_name, product_version=product_version
    )


def create_fluent_connection(
    channel, cleanup_on_exit: bool, instance, launcher_args: Dict[str, Any] | None
):
    """Create a Fluent connection."""

    return FluentConnection(
        channel=channel,
        cleanup_on_exit=cleanup_on_exit,
        remote_instance=instance,
        slurm_job_id=launcher_args.get("slurm_job_id") if launcher_args else None,
    )


def get_file_transfer_service(
    file_transfer_service: Any | None, fluent_connection
) -> Any:
    """Get the file transfer service."""

    return (
        file_transfer_service
        if file_transfer_service
        else PimFileTransferService(pim_instance=fluent_connection._remote_instance)
    )



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import pytest

from ansys.fluent.core import examples


@pytest.mark.nightly
@pytest.mark.fluent_version("==23.2")
def test_allowed_values_on_report_definitions_1364(new_solver_session):
    solver = new_solver_session

    import_file_name = examples.download_file(
        "elbow.cas.h5", "pyfluent/examples/DOE-ML-Mixing-Elbow"
    )

    solver.file.read(
        file_name=import_file_name, file_type="case", lightweight_setup=True
    )

    report_def = solver.solution.report_definitions.volume.create("xxx")

    report_def.set_state(
        {
            "report_type": "volume-max",
            "field": "temperature",
            "average_over": 1,
            "per_zone": False,
            "zone_names": ["fluid"],
            "expr_list": None,
        }
    )

    assert report_def.zone_names.allowed_values() == ["fluid"]

    assert report_def.expr_list.allowed_values() is None


@pytest.mark.fluent_version(">=23.2")
def test_monitors_list_set_data_637_974_1744_2188(new_solver_session):
    solver_session = new_solver_session

    import_case = examples.download_file(
        file_name="exhaust_system.cas.h5", directory="pyfluent/exhaust_system"
    )

    examples.download_file(
        file_name="exhaust_system.dat.h5", directory="pyfluent/exhaust_system"
    )

    solver_session.file.read_case_data(file_name=import_case)
    solver_session.solution.run_calculation.iterate(iter_count=1)

    monitors_list = solver_session.monitors.get_monitor_set_names()

    assert monitors_list == [
        "residual",
        "mass-bal-rplot",
        "mass-tot-rplot",
        "mass-in-rplot",
        "point-vel-rplot",
    ]

    mp = solver_session.monitors.get_monitor_set_data(monitor_set_name="residual")

    assert mp

    sample_report_plot = solver_session.solution.monitor.report_plots.create(
        "sample-report-plot"
    )
    sample_report_plot.report_defs = "mass-bal"

    solver_session.solution.initialization.hybrid_initialize()
    solver_session.solution.run_calculation.iterate(iter_count=1)

    new_monitors_list = solver_session.monitors.get_monitor_set_names()

    assert new_monitors_list == [
        "residual",
        "mass-bal-rplot",
        "mass-tot-rplot",
        "mass-in-rplot",
        "point-vel-rplot",
        "sample-report-plot",
    ]


@pytest.mark.fluent_version(">=24.2")
def test_empty_vector_field_data_2339(new_solver_session):
    solver = new_solver_session

    import_case = examples.download_file("mixing_elbow.cas.h5", "pyfluent/mixing_elbow")

    import_data = examples.download_file("mixing_elbow.dat.h5", "pyfluent/mixing_elbow")

    assert import_case
    assert import_data

    solver.file.read_case(file_name=import_case)

    solver.file.read_data(file_name=import_data)

    assert [
        a[0]
        for a in solver.fields.field_data.get_vector_field_data(
            field_name="velocity", surfaces=[1]
        )[1]
    ][:5]
